package dragon.rail.engine.graphics;

import android.graphics.Bitmap;
import dragon.rail.engine.Renderable;
import dragon.rail.engine.SafeArrayList;
import dragon.rail.engine.cel.Cel;
import dragon.rail.engine.shape.Rectangle;
import dragon.rail.sopwith.SopwithReduxView;
import dragon.rail.sopwith.util.Float2;

public class Sprite extends Renderable
{
  public SafeArrayList<Cel> cels;
  public int celFrame;

  public Sprite(SopwithReduxView gamePanel)
  {
    super(gamePanel);
    celFrame = 0;
    cels = new SafeArrayList<Cel>();
  }

  public Sprite(SopwithReduxView gamePanel, float x, float y)
  {
    this(gamePanel);
    position = new Float2(x, y);
    gamePanel.addRenderable(this);
  }

  /*public Sprite(SopwithReduxView gamePanel, CelHelper celHelper, int frame, float x, float y)
  {
    this(gamePanel, x, y);
    addCelsFromCelHelper(celHelper);
    celFrame = frame;
  }*/

  public Sprite(SopwithReduxView gamePanel, Cel cel, float x, float y)
  {
    this(gamePanel, x, y);
    cels.add(cel);
  }

  public String toString()
  {
    return name;
  }

  public Cel getCel()
  {
    return cels.get(celFrame);
  }

  public float getLeft()
  {
    return getCel().getGraphicsLeft() + position.x;
  }

  public float getTop()
  {
    return getCel().getGraphicsTop() + position.y;
  }

  public float getRight()
  {
    return getCel().getGraphicsRight() + position.x;
  }

  public float getBottom()
  {
    return getCel().getGraphicsBottom() + position.y;
  }

  public Bitmap getBufferedImage()
  {
    return getCel().bufferedImage;
  }

  /*public void setLayer(int layer)
  {
    this.layer = layer;
  }*/

  /*public void addCelsFromCelHelper(CelHelper celHelper)
  {
    for (CelSource cel : celHelper.getCels())
    {
      cels.add(cel.cel);
    }
  }

  public void addCelsFromCelHelper(CelHelper celHelper, Integer... celOrder)
  {
    for (int i = 0; i < celOrder.length; i++)
    {
      int index = celOrder[i];
      Cel cel = celHelper.get(index);
      cels.add(cel);
    }
  }*/

  public void setCel(int frame, Cel cel)
  {
    cels.set(frame, cel);
  }

  public boolean isVisible()
  {
    return super.isVisible() && getCel() != null;
  }

  public int numCels()
  {
    return cels.size();
  }

  public void addBoundingBoxes(float inset)
  {
    for (int i = 0; i < cels.size(); i++)
    {
      Cel cel = cels.get(i);
      if (cel != null)
      {
        Float2 topLeft = new Float2();
        Float2 bottomRight = new Float2();

        topLeft.y = cel.getGraphicsTop() + inset;
        topLeft.x = cel.getGraphicsLeft() + inset;
        bottomRight.y = cel.getGraphicsBottom() - inset;
        bottomRight.x = cel.getGraphicsRight() - inset;

        setCollsion(i, new Rectangle(topLeft, bottomRight, position));
      }
      else
      {
        setCollsion(i, null);
      }
    }
  }

  public void render(SopwithReduxView gamePanel)
  {
    gamePanel.renderSprite(this);
  }
}
