package dragon.rail.sopwith;


import android.content.Context;
import android.content.SharedPreferences;

/**
 * Keeps track of a player profile and all the data that corresponds
 * to such a profile
 * 
 * @author Pascal
 *
 */


public class PlayerProfile {

	
	// twitter name
	String				twitterName;
	String				twitterPassword;
	
	// audio?
	boolean 			useMusic;
	boolean				useSFX;
	
	// Control type
	boolean				useTouchControl;
	
	int					personalBest;
	
	// keep track of whatsnew info
	int					lastWhatsNew;
	
	
	/**
	 * Main constructor, initialise a blank profile
	 */
	public PlayerProfile() {
		useTouchControl=false;
		
		personalBest=0;
	}

	
	
	/**
	 * Load the settings for this profile
	 *
	 * @param mFileName Filename of the preferences file to use
	 * @return Map object (basically all data that was saved to the specified profile file)
	 */
	public final void loadSettings(Context context, String mFileName) {
		SharedPreferences prefs;
		
		prefs = context.getSharedPreferences( mFileName, 0 );

		// get profile preferences and settings
		useMusic=prefs.getBoolean("usemusic", true);
		useSFX=prefs.getBoolean("usesfx",true);

		useTouchControl=prefs.getBoolean("usetouchcontrol",false);
		
		twitterName=prefs.getString("twittername", "");
		twitterPassword=prefs.getString("twitterpass", "");
		
		lastWhatsNew=prefs.getInt("whatsnew",0);
		
		personalBest=prefs.getInt("personalBest", 0);
	}

	
	
	/**
	 * Save the settings for this profile
	 */
	public final void saveSettings(Context context, String mFileName) {
		SharedPreferences prefs;
		prefs = context.getSharedPreferences( mFileName, 0 );
		
		SharedPreferences.Editor prefsEditor = prefs.edit();
		prefsEditor.clear();
		
		// save settings and preferences
		prefsEditor.putBoolean("usemusic", useMusic);
		prefsEditor.putBoolean("usesfx", useSFX);
		
		prefsEditor.putBoolean("usetouchcontrol",useTouchControl);
		

		prefsEditor.putString("twittername",twitterName);
		prefsEditor.putString("twitterpass", twitterPassword);
		
		prefsEditor.putInt("whatsnew",lastWhatsNew);
		
		prefsEditor.putInt("personalBest", personalBest);
		prefsEditor.commit();
		
		prefsEditor=null;
	}


	
	
	/**
	 * Reset the settings of this player (whipes it all out!)
	 * 
	 * @param context 
	 * @param mFileName Filename of profile settings
	 */
	
	public final void resetSettings(Context context, String mFileName) {
		SharedPreferences prefs;
		prefs = context.getSharedPreferences( mFileName, 0 );
		
		SharedPreferences.Editor prefsEditor = prefs.edit();
		prefsEditor.clear();
		prefsEditor.commit();
		prefsEditor=null;
	}
	
	
	
	
	
}
