package dragon.rail.sopwith;

import java.util.ArrayList;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.view.MotionEvent;
import android.view.Surface.OutOfResourcesException;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import dragon.rail.engine.Camera;
import dragon.rail.engine.Renderable;
import dragon.rail.engine.graphics.Sprite;
import dragon.rail.sopwith.util.Float2;

public class SopwithReduxView extends SurfaceView {
	private Bitmap bmp;
	private SurfaceHolder holder;
	private SopwithReduxThread gameLoopThread;
	private Player player;
	private Controller upBtn;
	private Controller downBtn;
	private Controller bulletBtn;
	protected ArrayList<Renderable> activeObjects;
	protected Camera camera;

	private boolean drawUpCircle = false;
	private boolean drawDownCircle = false;
	// pr

	private long lastClick;

	// float cWidth = 0;
	// float cHeight = 0;
	float touchX = 0;
	float touchY = 0;

	// Float2 upBtnPos;
	// Float2 downBtnPos;

	// private Bitmap upBtn;
	/*
	 * private int handleRadius; private int handleInnerBoundaries;
	 */

	public SopwithReduxView(Context context) {

		super(context);

		activeObjects = new ArrayList<Renderable>();
		camera = new Camera(this, 0, 0);

		gameLoopThread = new SopwithReduxThread(this);
		holder = getHolder();
		holder.addCallback(new SurfaceHolder.Callback() {
			public void surfaceDestroyed(SurfaceHolder holder) {

				boolean retry = true;
				gameLoopThread.setRunning(false);
				while (retry) {
					try {
						gameLoopThread.join();
						retry = false;
					} catch (InterruptedException e) {
					}
				}
			}

			public void surfaceCreated(SurfaceHolder holder) {

				gameLoopThread.setRunning(true);
				gameLoopThread.start();

				// upBtnPos = new Float2(holder.getSurfaceFrame().height() -
				// 200, 30);
				// downBtnPos = new Float2(holder.getSurfaceFrame().height() -
				// 100, 30);

				bmp = BitmapFactory.decodeResource(getResources(),
						R.drawable.up_btn);
				upBtn = new Controller(bmp, new Float2(holder.getSurfaceFrame()
						.height() - 200, 30));

				bmp = BitmapFactory.decodeResource(getResources(),
						R.drawable.down_btn);
				downBtn = new Controller(bmp, new Float2(holder
						.getSurfaceFrame().height() - 100, 30));
			}

			public void surfaceChanged(SurfaceHolder holder, int format,
					int width, int height) {

			}
		});

		bmp = BitmapFactory.decodeResource(getResources(),
				R.drawable.sopwith_sprite2);
		player = new Player(this, bmp);

	}

	@Override
	protected void onDraw(Canvas canvas) {

		canvas.drawColor(Color.BLACK);
		player.onDraw(canvas);

		Paint paint = new Paint();
		paint.setAntiAlias(true);
		paint.setColor(Color.WHITE);
		paint.setStyle(Paint.Style.STROKE);

		/*
		 * canvas.drawCircle(upBtnPos.y, upBtnPos.x, 50, paint);
		 * canvas.drawCircle(downBtnPos.y, downBtnPos.x, 50, paint);
		 */

		if (drawUpCircle)
			canvas.drawCircle((upBtn.getPosition().y + upBtn.getRadius()),
					(upBtn.getPosition().x + upBtn.getRadius()),
					upBtn.getRadius(), paint);
		if (drawDownCircle)
			canvas.drawCircle((downBtn.getPosition().y + downBtn.getRadius()),
					(downBtn.getPosition().x + downBtn.getRadius()),
					downBtn.getRadius(), paint);

		canvas.drawBitmap(upBtn.getBmp(), upBtn.getPosition().y,
				upBtn.getPosition().x, null);
		canvas.drawBitmap(downBtn.getBmp(), downBtn.getPosition().y,
				downBtn.getPosition().x, null);

		// canvas.drawCircle((int) touchX + px, (int) touchY + py, handleRadius,
		// paint);
		drawUpCircle = false;
		drawDownCircle = false;
	}

	public boolean onTouchEvent(MotionEvent event) {

		if (System.currentTimeMillis() - lastClick > 300) {
			lastClick = System.currentTimeMillis();
			synchronized (getHolder()) {
				touchX = event.getX();
				touchY = event.getY();

				if (testing((upBtn.getPosition().x + upBtn.getRadius()),
						(upBtn.getPosition().y + upBtn.getRadius()), touchX,
						touchY, upBtn.getWidth() / 2)) {
					System.out.println("UP button HIT!!!");
					player.playerUp();
					drawUpCircle = true;
				}

				if (testing((downBtn.getPosition().x + downBtn.getRadius()),
						(downBtn.getPosition().y + downBtn.getRadius()),
						touchX, touchY, upBtn.getWidth() / 2)) {
					System.out.println("DOWN button HIT!!!");
					player.playerDown();
					drawDownCircle = true;
				}
			}
		}
		return true;
	}

	private boolean testing(float xCenter, float yCenter, float touchX,
			float touchY, int radius) {

		double distance;
		distance = Math.sqrt(Math.pow(xCenter - touchY, 2)
				+ (Math.pow(yCenter - touchX, 2)));
		if (distance <= radius) {
			return true;
		} else {
			return false;
		}
	}

	public void renderSprite(Sprite sprite) {
		int camX = (int) camera.getPosition().x;
		int camY = (int) camera.getPosition().y;
		int height = holder.getSurfaceFrame().height();
		int width = holder.getSurfaceFrame().width();
		int x = (int) (sprite.getLeft() - camX);
		int y = (int) (sprite.getTop() - camY);
		Rect rect = new Rect(camX, camY, height, width);
		// holder.getSurface().drawImage(sprite.getBufferedImage(), x, y, null);
		// this.draw(canvas)
		try {
			holder.getSurface().lockCanvas(rect)
					.drawBitmap(sprite.getBufferedImage(), x, y, null);
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (OutOfResourcesException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void addRenderable(Renderable sprite) {
		activeObjects.add(sprite);
	}
}