package dragon.rail.sopwith.util;

public class Angle
{
	
	public static final float myPI = 3.141592653589793f;
  public static float LEFT = (float) -(myPI / 2);
  public static float RIGHT = (float) (myPI / 2);
  public static float UP = 0;
  public static float DOWN = (float) myPI;

  public static float directionToAngle(Float2 fireDirection)
  {
    if (fireDirection.isZero())
    {
      return UP;
    }

    float angle = (float) Math.asin(fireDirection.x);
    if (fireDirection.y > 0)
    {
      if (fireDirection.x < 0)
      {
        angle = (float) -(angle + myPI);
      }
      else
      {
        angle = (float) (myPI - angle);
      }
    }

    return angle;
  }

  public static Float2 directionFrom(boolean right, boolean left, boolean up, boolean down)
  {
    Float2 direction = new Float2();
    if (right)
    {
      direction.x += 1.0f;
    }
    if (left)
    {
      direction.x -= 1.0f;
    }
    if (up)
    {
      direction.y -= 1.0f;
    }
    if (down)
    {
      direction.y += 1.0f;
    }

    direction.normalize();
    return direction;
  }

  public static float degToRad(final float degrees)
  {
    return (float) ((myPI * degrees) / 180.0);
  }

  public static float radToDeg(final float radians)
  {
    return (float) ((180.0 * radians) / myPI);
  }

  public static float rotateTowards(float requiredAngle, float currentAngle, float maxChange)
  {
    float angleBetween = angleBetween(requiredAngle, currentAngle);

    if (angleBetween > maxChange)
    {
      return currentAngle + maxChange;
    }
    else if (angleBetween < -maxChange)
    {
      return currentAngle - maxChange;
    }
    else
    {
      return requiredAngle;
    }
  }

  private static float angleBetween(float requiredAngle, float currentAngle)
  {
    requiredAngle = normaliseAngle(requiredAngle);
    currentAngle = normaliseAngle(currentAngle);

    return normaliseAngle(requiredAngle - currentAngle);
  }

  public static float normaliseAngle(float angle)
  {
    if (angle < -myPI)
    {
      int _180s = ((int) (-angle / (myPI)));
      if (_180s % 2 == 1)
      {
        _180s++;
      }
      float remove = _180s * (myPI);
      return angle + remove;
    }
    else if (angle > myPI)
    {
      int _180s = ((int) (angle / (myPI)));
      if (_180s % 2 == 1)
      {
        _180s++;
      }
      float remove = _180s * (myPI);
      return angle - remove;
    }
    else
    {
      return angle;
    }
  }

  public static Float2 angleToVector(float angle, float magnitude)
  {
    return new Float2((float) (Math.sin(angle) * magnitude), (float) -(Math.cos(angle) * magnitude));
  }

  public static boolean isLeftHalf(float angle)
  {
    angle = normaliseAngle(angle);
    if (angle < 0)
    {
      return true;
    }
    return false;
  }

  public static boolean isRightHalf(float angle)
  {
    angle = normaliseAngle(angle);
    if ((angle > 0) && (angle < DOWN))
    {
      return true;
    }
    return false;
  }

  public static boolean isTopHalf(float angle)
  {
    angle = normaliseAngle(angle);
    if ((angle > LEFT) && (angle < RIGHT))
    {
      return true;
    }
    return false;
  }

  public static boolean isBottomHalf(float angle)
  {
    angle = normaliseAngle(angle);
    if ((angle > RIGHT) && (angle <= DOWN))
    {
      return true;
    }
    if ((angle > LEFT) && (angle <= -DOWN))
    {
      return true;
    }
    return false;
  }

  public static int getFrameForAngle(float angle, int firstFrame, int lastFrame, int upFrame, boolean clockwise)
  {
    angle = normaliseAngle(angle);
    if (angle < 0)
    {
      angle = (myPI * 2) + angle;
    }
    float unit = (float) (angle / (myPI * 2));
    int frames = (lastFrame - firstFrame) + 1;

    unit += (0.5f / frames);
    int frame = (int) (frames * unit);
    frame += upFrame;
    frame += firstFrame;
    if (frame >= frames)
    {
      frame -= frames;
    }
    return frame;
  }
}
